@@ -40,6 +40,7 @@
 #include <sys/types.h>
 #include <sys/resource.h>
 #include <sys/stat.h>
+#include <sys/resource.h>
 #include <signal.h>
 #include <pwd.h>
 #include <grp.h>
@@ -50,6 +51,7 @@
 #include "vicious.h"
 
 #include "gdm.h"
+#include "gdmconfig.h"
 #include "verify.h"
 #include "gdm-net.h"
 #include "misc.h"
@@ -105,18 +107,15 @@ static gchar *GdmRemoteGreeter = NULL;
 static gchar *GdmGtkModulesList = NULL;
 static gchar *GdmChooser = NULL;
 static gchar *GdmLogDir = NULL;
-static gchar *GdmDisplayInit = NULL;
+static gchar *GdmDisplayInitDir = NULL;
 static gchar *GdmPostLogin = NULL;
 static gchar *GdmPreSession = NULL;
 static gchar *GdmPostSession = NULL;
 static gchar *GdmFailsafeXserver = NULL;
 static gchar *GdmXKeepsCrashing = NULL;
 static gchar *GdmHalt = NULL;
-static gchar *GdmHaltReal = NULL;
 static gchar *GdmReboot = NULL;
-static gchar *GdmRebootReal = NULL;
 static gchar *GdmSuspend = NULL;
-static gchar *GdmSuspendReal = NULL;
 static gchar *GdmServAuthDir = NULL;
 static gchar *GdmMulticastAddr;
 static gchar *GdmUserAuthDir = NULL;
@@ -136,6 +135,7 @@ static gchar *GdmSoundOnLoginFile = NULL;
 static gchar *GdmSoundOnLoginSuccessFile = NULL;
 static gchar *GdmSoundOnLoginFailureFile = NULL;
 static gchar *GdmConsoleCannotHandle = NULL;
+static gchar *GdmPamStack = NULL;
 
 static gint GdmXineramaScreen = 0;
 static gint GdmUserMaxFile = 0;
@@ -195,31 +195,29 @@ static gboolean GdmConsoleNotify = TRUE;
 
 /* Config options used by slave */
 /* ---------------------------- */
-static gchar *GdmInitDir;
-static gchar *GdmGtkRc;
-static gchar *GdmGtkThemesToAllow;
-static gchar *GdmInclude;
-static gchar *GdmExclude;
-static gchar *GdmDefaultFace;
-static gchar *GdmLocaleFile;
-static gchar *GdmLogo;
-static gchar *GdmChooserButtonLogo;
-static gchar *GdmWelcome;
-static gchar *GdmRemoteWelcome;
-static gchar *GdmBackgroundProgram;
-static gchar *GdmBackgroundImage;
-static gchar *GdmBackgroundColor;
-static gchar *GdmGraphicalTheme;
-static gchar *GdmInfoMsgFile;
-static gchar *GdmInfoMsgFont;
-static gchar *GdmHost;
-static gchar *GdmHostImageDir;
-static gchar *GdmHosts;
-static gchar *GdmGraphicalThemeColor;
-static gchar *GdmGraphicalThemeDir;
-static gchar *GdmGraphicalThemes;
-static gchar *GdmPreFetchProgram;
-static gchar *GdmUse24Clock;
+static gchar *GdmGtkThemesToAllow = NULL;
+static gchar *GdmInclude = NULL;
+static gchar *GdmExclude = NULL;
+static gchar *GdmDefaultFace = NULL;
+static gchar *GdmLocaleFile = NULL;
+static gchar *GdmLogo = NULL;
+static gchar *GdmChooserButtonLogo = NULL;
+static gchar *GdmWelcome = NULL;
+static gchar *GdmRemoteWelcome = NULL;
+static gchar *GdmBackgroundProgram = NULL;
+static gchar *GdmBackgroundImage = NULL;
+static gchar *GdmBackgroundColor = NULL;
+static gchar *GdmGraphicalTheme = NULL;
+static gchar *GdmInfoMsgFile = NULL;
+static gchar *GdmInfoMsgFont = NULL;
+static gchar *GdmHost = NULL;
+static gchar *GdmHostImageDir = NULL;
+static gchar *GdmHosts = NULL;
+static gchar *GdmGraphicalThemeColor = NULL;
+static gchar *GdmGraphicalThemeDir = NULL;
+static gchar *GdmGraphicalThemes = NULL;
+static gchar *GdmPreFetchProgram = NULL;
+static gchar *GdmUse24Clock = NULL;
 
 static gint GdmPositionX;
 static gint GdmPositionY;
@@ -436,7 +434,7 @@ gdm_config_init (void)
    gdm_config_add_hash (GDM_KEY_X_KEEPS_CRASHING, &GdmXKeepsCrashing, &string_type);
    gdm_config_add_hash (GDM_KEY_BASE_XSESSION, &GdmBaseXsession, &string_type);
    gdm_config_add_hash (GDM_KEY_REMOTE_GREETER, &GdmRemoteGreeter, &string_type);
-   gdm_config_add_hash (GDM_KEY_DISPLAY_INIT_DIR, &GdmInitDir, &string_type);
+   gdm_config_add_hash (GDM_KEY_DISPLAY_INIT_DIR, &GdmDisplayInitDir, &string_type);
    gdm_config_add_hash (GDM_KEY_AUTOMATIC_LOGIN, &GdmAutomaticLogin, &string_type);
    gdm_config_add_hash (GDM_KEY_GTK_MODULES_LIST, &GdmGtkModulesList, &string_type);
    gdm_config_add_hash (GDM_KEY_REBOOT, &GdmReboot, &string_type);
@@ -455,7 +453,7 @@ gdm_config_init (void)
    gdm_config_add_hash (GDM_KEY_MULTICAST_ADDR, &GdmMulticastAddr, &string_type);
    gdm_config_add_hash (GDM_KEY_USER, &GdmUser, &string_type);
    gdm_config_add_hash (GDM_KEY_GROUP, &GdmGroup, &string_type);
-   gdm_config_add_hash (GDM_KEY_GTKRC, &GdmGtkRc, &string_type);
+   gdm_config_add_hash (GDM_KEY_GTKRC, &GdmGtkRC, &string_type);
    gdm_config_add_hash (GDM_KEY_GTK_THEME, &GdmGtkTheme, &string_type);
    gdm_config_add_hash (GDM_KEY_TIMED_LOGIN, &GdmTimedLogin, &string_type);
    gdm_config_add_hash (GDM_KEY_WILLING, &GdmWilling, &string_type);
@@ -504,6 +502,7 @@ gdm_config_init (void)
    gdm_config_add_hash (GDM_KEY_HOSTS, &GdmHosts, &string_type);
    gdm_config_add_hash (GDM_KEY_PRE_FETCH_PROGRAM,
       &GdmPreFetchProgram, &string_type);
+   gdm_config_add_hash (GDM_KEY_PAM_STACK, &GdmPamStack, &string_type);
 
    /* int values */
    gdm_config_add_hash (GDM_KEY_XINERAMA_SCREEN, &GdmXineramaScreen, &int_type);
@@ -609,6 +608,19 @@ gdm_get_custom_config (struct stat *statbuf)
    }
 }
 
+/**
+ * gdm_get_per_display_custom_config_file
+ *
+ * Returns the per-display config file for a given display
+ * This is always the custom config file name with the display
+ * appended, and never gdm.conf.
+ */
+static gchar *
+gdm_get_per_display_custom_config_file (gchar *display)
+{
+  return g_strdup_printf ("%s%s", custom_config_file, display);
+}
+ 
 /**
  * gdm_get_custom_config_file
  *
@@ -704,6 +716,171 @@ gdm_get_value_bool (char *key)
    return FALSE;
 }
 
+/**
+ * Note that some GUI configuration parameters are read by the daemon,
+ * and in order for them to work, it is necessary for the daemon to 
+ * access a few keys in a per-display fashion.  These access functions
+ * allow the daemon to access these keys properly.
+ */
+
+/**
+ * gdm_get_value_int_per_display
+ *
+ * Gets the per-display version  of the configuration, or the default
+ * value if none exists.
+ */
+gint gdm_get_value_int_per_display (gchar *display, gchar *key)
+{
+   gchar *perdispval;
+
+   gdm_config_key_to_string_per_display (display, key, &perdispval);
+
+   if (perdispval != NULL) {
+      gint val = atoi (perdispval);
+      g_free (perdispval);
+      return val;
+   } else {
+      return (gdm_get_value_int (key));
+   }
+}
+ 
+/**
+ * gdm_get_value_bool_per_display
+ *
+ * Gets the per-display version  of the configuration, or the default
+ * value if none exists.
+ */
+gboolean gdm_get_value_bool_per_display (gchar *display, gchar *key)
+{
+   gchar *perdispval;
+
+   gdm_config_key_to_string_per_display (display, key, &perdispval);
+
+   if (perdispval != NULL) {
+      if (perdispval[0] == 'T' ||
+          perdispval[0] == 't' ||
+          perdispval[0] == 'Y' ||
+          perdispval[0] == 'y' ||
+          atoi (perdispval) != 0) {
+		g_free (perdispval);
+		return TRUE;
+       } else {
+		return FALSE;
+       }
+   } else {
+      return (gdm_get_value_bool (key));
+   }
+}
+ 
+/**
+ * gdm_get_value_string_per_display
+ *
+ * Gets the per-display version  of the configuration, or the default
+ * value if none exists.  Note that this value needs to be freed,
+ * unlike the non-per-display version.
+ */
+gchar * gdm_get_value_string_per_display (gchar *display, gchar *key)
+{
+   gchar *perdispval;
+
+   gdm_config_key_to_string_per_display (display, key, &perdispval);
+
+   if (perdispval != NULL) {
+      return perdispval;
+   } else {
+      return (g_strdup (gdm_get_value_string (key)));
+   }
+}
+
+/**
+ * gdm_config_key_to_string_per_display
+ *
+ * If the key makes sense to be per-display, return the value,
+ * otherwise return NULL.  Keys that only apply to the daemon
+ * process do not make sense for per-display configuration  
+ * Valid keys include any key in the greeter or gui categories,
+ * and the GDM_KEY_PAM_STACK key.
+ *
+ * If additional keys make sense for per-display usage, make
+ * sure they are added to the if-test below.
+ */
+void
+gdm_config_key_to_string_per_display (gchar *display, gchar *key, gchar **retval)
+{
+   gchar *file;
+   gchar **splitstr = g_strsplit (key, "/", 2);
+   *retval = NULL;
+
+   if (display == NULL)
+      return;
+
+   file = gdm_get_per_display_custom_config_file (display);
+
+   if (strcmp (ve_sure_string (splitstr[0]), "greeter") == 0 ||
+       strcmp (ve_sure_string (splitstr[0]), "gui") == 0 ||
+       is_key (key, GDM_KEY_PAM_STACK)) {
+      gdm_config_key_to_string (file, key, retval);
+   }
+
+   g_free (file);
+
+   return;
+}
+
+/**
+ * gdm_config_key_to_string
+ *
+ * Returns a specific key from the config file, or NULL if not found.
+ * Note this returns the value in string form, so the caller needs
+ * to parse it properly if it is a bool or int.
+ */
+void
+gdm_config_key_to_string (gchar *file, gchar *key, gchar **retval)
+{
+   VeConfig *cfg = ve_config_get (file);
+   GdmConfigType *type = gdm_config_hash_lookup (type_hash, key);
+   gchar **splitstr = g_strsplit (key, "/", 2);
+   *retval = NULL;
+
+   /* Should not fail, all keys should have a category. */
+   if (splitstr[0] == NULL)
+      return;
+
+   /* If file doesn't exist, then just return */
+   if (cfg == NULL)
+      return;
+
+   GList *list = ve_config_get_keys (cfg, splitstr[0]);
+   while (list != NULL) {
+      gchar *display_key     = (char *)list->data;
+      gchar *display_fullkey = g_strdup_printf ("%s/%s", splitstr[0], display_key);
+
+      if (is_key (key, display_fullkey)) {
+         gdm_debug ("Returning value for key <%s>\n", key);
+         if (*type == CONFIG_BOOL) {
+            gboolean value = ve_config_get_bool (cfg, key);
+            if (value)
+               *retval = g_strdup ("true");
+            else
+               *retval = g_strdup ("false");
+            return;
+         } else if (*type == CONFIG_INT) {
+            gint value = ve_config_get_int (cfg, key);
+            *retval = g_strdup_printf ("%d", value);
+            return;
+         } else if (*type == CONFIG_STRING) {
+            gchar *value = ve_config_get_string (cfg, key);
+            if (value != NULL)
+               *retval = g_strdup (value);
+            return;
+         }
+      }
+      g_free (display_fullkey);
+      list = list->next;
+   }
+   return;
+}
+
 /**
  * gdm_config_to_string
  *
@@ -711,9 +888,20 @@ gdm_get_value_bool (char *key)
  * GET_CONFIG socket command.
  */ 
 void
-gdm_config_to_string (gchar *key, gchar **retval)
+gdm_config_to_string (gchar *key, gchar *display, gchar **retval)
 {
-   GdmConfigType *type;
+   GdmConfigType *type = gdm_config_hash_lookup (type_hash, key);
+   *retval = NULL;
+
+   /*
+    * See if there is a per-display config file, returning that value
+    * if it exists.
+    */
+   if (display) {
+      gdm_config_key_to_string_per_display (display, key, retval);
+      if (*retval != NULL)
+         return;
+   }
 
    /* First look in translated_hash */
    if (translated_hash != NULL) {
@@ -724,9 +912,6 @@ gdm_config_to_string (gchar *key, gchar **retval)
       }
    }
 
-   type   = gdm_config_hash_lookup (type_hash, key);
-   *retval = NULL;
-
    if (type != NULL) {
       if (*type == CONFIG_BOOL) {
          gboolean value = gdm_get_value_bool (key);
@@ -1105,7 +1290,7 @@ gdm_set_value_bool (gchar *key, gboolean value)
    _gdm_set_value_bool (key, value, TRUE);
 }
 
-void
+static void
 _gdm_set_value_int (gchar *key, gint value, gboolean doing_update)
 {
    gint *setting     = gdm_config_hash_lookup (val_hash, key);
@@ -1448,7 +1633,7 @@ gdm_update_config (gchar* key)
    struct stat statbuf, custom_statbuf;
    VeConfig *cfg;
    VeConfig *custom_cfg = NULL;
-   gboolean rc;
+   gboolean rc = FALSE;
 
    /*
     * Do not allow these keys to be updated, since GDM would need
@@ -1508,7 +1693,7 @@ gdm_update_config (gchar* key)
       ve_config_destroy (cfg);
       if (custom_cfg != NULL)
          ve_config_destroy (custom_cfg);
-      return rc;
+      return TRUE;
    }
 
    type = gdm_config_hash_lookup (type_hash, key);
@@ -1695,7 +1880,7 @@ gdm_load_displays (VeConfig *cfg, GList *list )
  *
  * Called by gdm_config_parse in a loop to set each key.
  */
-void
+static void
 gdm_load_config_option (gpointer key_in, gpointer value_in, gpointer data)
 {
    gchar *key               = (gchar *)key_in;
@@ -2067,39 +2252,6 @@ gdm_set_high_display_num (gint val)
    high_display_num = val;
 }
 
-/**
- * gdm_print_config_option
- *
- * Called by gdm_print_all_config in a loop to print each key.
- * 
- */
-void
-gdm_print_config_option (gpointer key_in, gpointer value_in, gpointer data)
-{
-   gchar *key = (gchar *)key_in;
-   gchar *retval;
-
-   gdm_config_to_string (key, &retval);
-   if (retval != NULL) {
-      gdm_debug ("key is %s, value is %s\n", key, retval);
-      g_free (retval);
-   } else
-      gdm_debug ("key is %s, value is <NULL>\n", key);
-}
-
-/**
- * gdm_print_all_config
- *
- * Not used by GDM, but useful for debug purposes.
- */
-void
-gdm_print_all_config (void)
-{
-   gdm_config_parse ();
-
-   g_hash_table_foreach (type_hash, gdm_print_config_option, NULL);
-}
-
 /**
  * gdm_is_valid_key
  *
