@@ -162,7 +162,8 @@ greeter_language_get_language (const char *old_language)
 	  g_free (current_name);
 	  g_free (saved_name);
 
-	  savelang = gdm_common_query (primary_message, secondary_message, _("Make _Default"), _("Just For _This Session"), TRUE);
+	  savelang = gdm_wm_query_dialog (primary_message, secondary_message,
+		_("Make _Default"), _("Just For _This Session"), TRUE);
 	  g_free (primary_message);
 	  g_free (secondary_message);
 	}
@@ -205,10 +206,16 @@ tree_row_activated (GtkTreeView         *view,
   }
 }
 
+/*
+ * The button with this handler appears in the F10 menu, so it
+ * cannot depend on callback data being passed in.
+ */
 void
-greeter_action_language (GreeterItemInfo *info,
-			 gpointer         user_data)
+greeter_language_handler (void)
 {
+  GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
+  GtkWidget *entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
+
   GtkWidget *view = NULL;
 
   if (dialog == NULL)
@@ -327,5 +334,8 @@ greeter_action_language (GreeterItemInfo *info,
 
   if (dialog)
     gtk_widget_hide (dialog);
+
+  /* Make sure entry has focus after button press */
+  gtk_widget_grab_focus (entry);
 }
 
