@@ -19,6 +19,7 @@
 #include "config.h"
 
 #include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include <librsvg/rsvg.h>
 #include "gdmcommon.h"
 #include "gdmwm.h"
@@ -166,7 +167,7 @@ greeter_size_allocate_fixed (GreeterItemInfo *fixed,
 
       if (child->item_type != GREETER_ITEM_TYPE_LABEL)
 	fixup_from_anchor (&child_allocation, child->anchor);
-      
+
       greeter_item_size_allocate (child,
 				  &child_allocation,
 				  canvas);
@@ -559,6 +560,34 @@ greeter_item_size_request (GreeterItemInfo *item,
       req->height = gdk_pixbuf_get_height (svg);
       g_object_unref (svg);
     }
+
+  if (item->item_type == GREETER_ITEM_TYPE_BUTTON)
+    {
+#define ITEM_BUTTON_MIN_RECOMMANDED_WIDTH_OFFSET 15      
+#define ITEM_BUTTON_MIN_RECOMMANDED_HEIGHT_OFFSET 10
+      PangoLayout *layout;
+      int pango_width, pango_height;
+      int pix_width, pix_height;
+      
+      GtkWidget *dummy_w = gtk_button_new ();
+      
+      layout = gtk_widget_create_pango_layout (dummy_w, item->data.text.orig_text);
+       
+      pango_layout_get_size (layout, &pango_width, &pango_height);
+      
+      pix_height = PANGO_PIXELS (pango_height) + ITEM_BUTTON_MIN_RECOMMANDED_HEIGHT_OFFSET;
+      pix_width = PANGO_PIXELS (pango_width) + ITEM_BUTTON_MIN_RECOMMANDED_WIDTH_OFFSET;
+
+      if (pix_width > item->parent->box_min_width)
+	req->width = pix_width;
+      else
+	req->width = item->parent->box_min_width;
+
+      if (pix_height > item->parent->box_min_height)
+	req->height = pix_height;
+      else
+	req->height = item->parent->box_min_height;
+    }
  
   if (req->width > 0 && req->height > 0)
     {
