@@ -28,6 +28,8 @@
 #include "greeter_configuration.h"
 #include "greeter_system.h"
 #include "greeter_item.h"
+#include "greeter_item_ulist.h"
+#include "greeter_parser.h"
 #include "gdm.h"
 #include "gdmcommon.h"
 #include "gdmconfig.h"
@@ -68,35 +70,46 @@ bin_exists (const char *command)
 static void
 query_greeter_restart_handler (void)
 {
+       GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
+       GtkWidget *entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
+
 	if (gdm_wm_warn_dialog (_("Are you sure you want to restart the computer?"), "",
 			     _("_Restart"), NULL, TRUE) == GTK_RESPONSE_YES) {
 		closelog ();
 		
 		_exit (DISPLAY_REBOOT);
 	}
+	gtk_widget_grab_focus (entry);
 }
 
-
 static void
 query_greeter_halt_handler (void)
 {
+	GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
+	GtkWidget *entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
+
 	if (gdm_wm_warn_dialog (_("Are you sure you want to Shut Down the computer?"), "",
 			     _("Shut _Down"), NULL, TRUE) == GTK_RESPONSE_YES) {
 		closelog ();
 
 		_exit (DISPLAY_HALT);
 	}
+	gtk_widget_grab_focus (entry);
 }
 
 static void
 query_greeter_suspend_handler (void)
 {
+	GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
+	GtkWidget *entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
+
 	if (gdm_wm_warn_dialog (_("Are you sure you want to suspend the computer?"), "",
 			     _("_Suspend"), NULL, TRUE) == GTK_RESPONSE_YES) {
 		/* suspend interruption */
 		printf ("%c%c%c\n", STX, BEL, GDM_INTERRUPT_SUSPEND);
 		fflush (stdout);
 	}
+	gtk_widget_grab_focus (entry);
 }
 
 static void
@@ -124,6 +137,11 @@ greeter_suspend_handler (void)
 static void
 greeter_config_handler (void)
 {
+	greeter_item_ulist_disable ();
+
+	/* Make sure to unselect the user */
+	greeter_item_ulist_unset_selected_user ();
+
 	/* we should be now fine for focusing new windows */
 	gdm_wm_focus_new_windows (TRUE);
 
